﻿using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using Eingabeverwaltung.Metamodell;

namespace Bewertung
{
    public class AssessmentObject
    {
        public readonly SingleTask task;
        public readonly List<Lemma> usefulLemmata;
        internal double assessment;
        internal ImmutableList<InductionPartAssessment> confIndAssessment;
        public ImmutableDictionary<Lemma, int> AbstractMaxLemmaPoints;

        
        /// <param name="task"></param>
        /// <param name="usefulLemmata">first useful lemma should be the main lemma of the task</param>
        public AssessmentObject(SingleTask task, List<Lemma> usefulLemmata)
        {
            this.task = task;
            this.usefulLemmata = usefulLemmata;
            assessment = 0.0;
            // init lemmaAbstractPoints
            Dictionary<Lemma, int> AbstractMaxLemmaPoints = new Dictionary<Lemma, int>();
            foreach (Lemma lemma in usefulLemmata)
                if (lemma.proof is Transformation tr)
                    AbstractMaxLemmaPoints.Add(lemma, tr.transformationSteps.Count);
                else if (lemma.proof is Induction ind)
                    AbstractMaxLemmaPoints.Add(lemma, lemmaAbstractPoints(ind));
            this.AbstractMaxLemmaPoints = AbstractMaxLemmaPoints.ToImmutableDictionary();
            // init confIndAssessment
            List<InductionPartAssessment> confIndAssessment = new List<InductionPartAssessment>();
            if (task.config is InductionConfig indConf)
            {
                foreach (InductionCaseConfig icC in indConf.indCaseConfigs)
                    if (icC.indCase is BasicConstructor)
                        confIndAssessment.Add(new InductionPartAssessment(icC.config.maxPt, "Fall " + icC.indCase.name));
                confIndAssessment.Add(new InductionPartAssessment(indConf.indHypConfig.maxPt, "IH"));
                foreach (InductionCaseConfig icC in indConf.indCaseConfigs)
                    if (icC.indCase is BuildingConstructor)
                        confIndAssessment.Add(new InductionPartAssessment(icC.config.maxPt, "Fall " + icC.indCase.name));
            }
            this.confIndAssessment = confIndAssessment.ToImmutableList();
        }

        internal void init()
        {
            assessment = 0.0;
            foreach (InductionPartAssessment ipa in confIndAssessment)
                ipa.Assessment = 0.0;
        }

        private int lemmaAbstractPoints(Induction ind)
        {
            int result = 1; // 1 for ih
            foreach (InductionCase ic in ind.indCases)
            {
                bool err33 = false;
                foreach (Error err in ic.errors)
                    if (err.errorCode == 33) { err33 = true; break; } // invalid ind cases are not assessed
                if (!err33)
                    result += ic.proof.transformationSteps.Count;
            }
            return result;
        }

        /// <summary>
        /// Returns the maximum abstract points of the single Task of this AssessmentObject
        /// </summary>
        /// <returns></returns>
        public double AbstractMaxTaskPoints() 
        {
            double result = 0.0; ImmutableDictionary<Lemma, int> abstrmlp = AbstractMaxLemmaPoints;
            if (task.config is InductionConfig)
                abstrmlp = abstrmlp.Remove(usefulLemmata[0]); // because its immutable it don't destroys AbstractMaxLemmaPoints
            foreach (var abstractMaxLemmaPoints in abstrmlp)
                result += (double)abstractMaxLemmaPoints.Value / abstractMaxLemmaPoints.Key.usefulFor.Count;
            return result;
        }

        internal InductionPartAssessment findIPA(string name)
        {
            foreach (InductionPartAssessment ipa in confIndAssessment)
                if (ipa.partID == name) return ipa;
            throw new Exception("Could not find InductionPartAssessment " + name);
        }
    }
}
