﻿using System.Collections.Generic;

namespace Eingabeverwaltung.Metamodell
{
	public class Induction : SingleProof
	{
		public readonly Equation       eq;
		public readonly List<Variable> fixedVars;
		public readonly Variable            indVar;
		public readonly List<InductionCase> indCases;
		public readonly List<InductionHypothesis> indHyps;
		public readonly List<Error> errors;
		public Induction(Equation eq, List<Variable> fixedVars, Variable indVar, List<InductionCase> indCases, List<InductionHypothesis> indHyps)
		{
			this.eq = eq;
			this.fixedVars = fixedVars;
			this.indVar = indVar;
			this.indCases = indCases;
			this.indHyps = indHyps;
			errors = new List<Error>();
		}

		public List<InductionCase> GetIndBasises()
        {
			List<InductionCase> result = new List<InductionCase>();
			foreach (InductionCase ic in indCases)
				if (ic.type == "IA")
					result.Add(ic);
			return result;
		}

		public List<InductionCase> GetIndSteps()
		{
			List<InductionCase> result = new List<InductionCase>();
			foreach (InductionCase ic in indCases)
				if (ic.type == "IS")
					result.Add(ic);
			return result;
		}

		public override string ToString(bool showMetadata)
        {
			string res = "Induction " + (indVar != null ? indVar.name : (showMetadata ? "IndVar not found because of error 22" : "")) + ":\r\n";
			foreach (InductionCase indBasis in GetIndBasises())
				res += indBasis.ToString(showMetadata) +"\r\n";
			foreach(InductionHypothesis ih in indHyps)
				res += ih.ToString();
			foreach (InductionCase indStep in GetIndSteps())
				res += indStep.ToString(showMetadata) +"\r\n";
			return res;
        }
    }
}
